{{ Modi 01.07.2007 Christof Eberspcher

From Parallax Inc. Propeller Education Kit - Counters & Circuits Lab   
CalibrateMetalDetector.spin

   This modified version finds the resonant frequency automatically.
   With "F" the minimum is looked for by gradual approximation.
   With "C" the minimum is found through statistical evaluation:
     First by the points on the right of and on the left of the minimum two straight lines are put with linear regression.
     Their intersection is the center frequency.
   With center frequency approx. 160kHz is the standard deviation of the "F" method 22Hz.
   With "C" method it is 15 cycles per second.
   In each case the search window must be determined first by hand.
   A super-simple diagram shows the shift of the frequency on the terminal.
   A coin in distance of about 60mm is detectable.

   Diese modifizierte Version findet die Resonanzfrequenz automatisch.
   Bei F wird das Minimum durch Sukzessive Approximation gesucht.
   Bei C wird das Minimum durch statistisch Auswertung gefunden:
     Zunchst werden durch die Punkte rechts und links des Minimums zwei Regeressionsgeraden gelegt.
     Deren Schnittpunkt ist die Mittenfrequenz.
   Bei Mittenfrequenz ca 160kHz ist die Standardabweichung der F- Methode 22Hz. Diejenige der C- Methode 15 Hz. 
   Jeweils muss das Suchfenster zunchst von Hand bestimmt werden.
   Eine super-simple Grafik zeigt die Frequenzverschiebung auf dem Terminal. 
   Eine Mnze in Abstand 60mm ist detektierbar.

}}

CON
   
  _clkmode = xtal1 + pll16x            ' Set up 80 MHz internal clock
  _xinfreq = 5_000_000
  Ausgang = 6               'Output
  Eingang = 7               'Input

OBJ                                   
   
  Debug   : "FullDuplexSerialPlus"
  frq     : "SquareWave"
  FM      : "FloatMath"
  FS      : "FloatString"


VAR
 long i, k
 long x[22], y[22],   a, b, a1, a2, b1, b2, SXY, SX, SY, SXQ, xs, help  'reals


PUB main
  Init

PUB Regression | j
   Reg(0)
   a1:= a
   b1:= b
   Reg(11)
   a2:= a
   b2:= b
   xs:= fm.fsub(a1,a2)          'Schnittpunkt
   help:= fm.fsub(b2,b1)
   xs:= fm.fdiv(xs,help)


PUB Reg(imin)
  sxy:= sx := sy:= sxq:= 0.0
  repeat i from imin to imin+10
    sx := fm.fadd(sx,x[i])
    sy := fm.fadd(sy,y[i])
    
    help:= fm.fmul(x[i],x[i])
    sxq:= fm.fadd(sxq, help)
   
    help:= fm.fmul(x[i],y[i])
    sxy:= fm.fadd(sxy, help)

  b:= fm.fmul(sx, sy)
  b:= fm.fdiv(b,11.0)
  b:= fm.fsub(sxy,b)

  help:= fm.fmul(sx,sx)
  help:= fm.fdiv(help,11.0)
  help:= fm.fsub(sxq,help)

  b:= fm.fdiv(b,help)

  a:= fm.fmul(b,sx)
  a:= fm.fsub(sy,a)
  a:= fm.fdiv(a,11.0)



  
 
PUB Init | count, f, fstart, fstep, c, fmin, cmin, t, fminm, steig, steigm, fsmax, oldcount

  'Start FullDuplexSerialPlus
  Debug.start(31, 30, 0, 57600)
  waitcnt(clkfreq*2 + cnt)

  'Configure ctra module for 50 MHz square wave
  ctra[30..26] := %00010
  ctra[25..23] := %110            
  ctra[8..0] := Ausgang               '15        
  frq.Freq(0, Ausgang, 50_000_000)                         
  dira[Ausgang]~~                     '15
  
  'Configure ctrb module for negative edge counting
  ctrb[30..26] := %01000               
  ctrb[8..0] := Eingang               '7
  frqb := 1

  fs.SetPrecision(10)
  
  c := "S"

  repeat' until c == "Q" or c == "q"

    case c
      "S", "s":
        Debug.Str(String(10, 13, "Starting Frequency: "))
        f := Debug.GetDec
        Debug.Str(String(10, 13, "Step size: "))
        fstep := Debug.GetDec
        Debug.Str(String(10, 13))
         
    case c
      "S", "s", 13, 10, "M", "m":
        repeat 22
          frq.Freq(0, Ausgang, f)   
          count := phsb
          waitcnt(clkfreq/10000 + cnt)                      
          count := phsb - count
          Debug.Str(String(10, 13, "Freq = "))
          Debug.Dec(f)
          Debug.Str(String("  count = "))
          Debug.Dec(count)
          repeat count*80/3000
            Debug.Str(String("="))
          waitcnt(clkfreq/20 + cnt)
          f += fstep
         
        Debug.str(String(10,13,"Enter->more, Q->Quit, S->Start over, R->repeat A-> Auto C-> Float Center F-> Find Min P -> Steilabfall: "))
        c := Debug.rx
        Debug.str(String(10, 13))

      "R", "r":
        f -= (22 * fstep)
        c := "m"


      "C", "c":
        f-=(22* fstep)
        fminm:= f+ 11*fstep
        k:=0
        repeat
                  k+=1
                  if k==3
                    fminm:= fmin
                  cmin:= 10000              
                  repeat i from 0 to 21 
                                frq.Freq(0, Ausgang, f)   
                                count := phsb
                                waitcnt(clkfreq/100 + cnt)                      
                                count := phsb - count
                                x[i]:= fm.ffloat(f)
                                y[i]:= fm.ffloat(count)
                                if count<cmin
                                  cmin:=count
                                  fmin:=f
                                {debug.Str(String(10, 13, "Freq = "))
                                debug.Dec(f)
                                debug.Str(String("  count = "))
                                debug.Dec(count)
                                repeat count*80/3000/4
                                 Debug.Str(String("="))
                                waitcnt(clkfreq/10 + cnt)}
                                f+= fstep

                  {debug.Str(String(10, 13, "Minf = "))
                  debug.Dec(fmin)
                  debug.Str(String("  count = "))
                  debug.Dec(cmin) }

                  Regression
                  debug.Str(String(10, 13, " S = "))
                  debug.Str(fs.floattostring(xs))
                  debug.Str(String(" S-Div = "))
                  debug.Str(fs.floattostring(fm.fsub(b1,b2)))                  
                  {debug.Str(String(10, 13))                    
                  debug.Str(fs.floattostring(a2))
                  debug.Str(String(10, 13))                    
                  debug.Str(fs.floattostring(b2))}                                    

 
                  fmin:= fm.ftrunc(xs)
                  fminm:= (59*fminm + fmin)/60
                  if (fmin-fminm)>-500 and (fmin-fminm)<500
                      repeat (fmin-fminm+500)*80/500
                         Debug.Str(String("="))
                  f := fmin -11*fstep                          
                  'waitcnt(cnt + clkfreq *2)
                    

       
      "A", "a":
        repeat
          f-=(22* fstep)
          
                  cmin:= 10000              
                  repeat 22
                                frq.Freq(0, Ausgang, f)   
                                count := phsb
                                waitcnt(clkfreq/10000 + cnt)                      
                                count := phsb - count
                                if count<cmin
                                  cmin:=count
                                  fmin:=f
                                debug.Str(String(10, 13, "Freq = "))
                                debug.Dec(f)
                                debug.Str(String("  count = "))
                                debug.Dec(count)
                                repeat count*80/3000
                                 Debug.Str(String("="))
                                waitcnt(clkfreq/10 + cnt)
                                f+= fstep

                  debug.Str(String(10, 13, 10, 13, "Minf = "))
                  debug.Dec(fmin)
                  debug.Str(String("  count = "))
                  debug.Dec(cmin)               
                  waitcnt(cnt + clkfreq *2)

    
      "P", "p":              ' Steilster Abfall
        
        f-=(11* fstep)        'Zentralfrequenz
        fminm:= f
        fstep*= 3      
        repeat  
          t:= 1
             repeat 7
               t:= t*2
               f-=3*fstep/t
               steigm:= 0
               frq.Freq(0, Ausgang, f)   
               count := phsb
               waitcnt(clkfreq/1000 + cnt)
               count := phsb - count         
                  repeat 6
                                oldcount:= count
                                frq.Freq(0, Ausgang, f)   
                                count := phsb
                                waitcnt(clkfreq/1000 + cnt)                      
                                count := phsb - count
                                steig:= oldcount-count
                                
                                if steig>steigm
                                  steigm:= steig
                                  fsmax:=f
                                  
                                '{  
                                debug.Str(String(10, 13, "Freq = "))
                                debug.Dec(f)
                                debug.Str(String("  count = "))
                                debug.Dec(count)
                                debug.Str(String("  steig = "))
                                debug.Dec(steig)
                                {
                                repeat count*80/500
                                  Debug.Str(String("="))
                                  
                                waitcnt(clkfreq/10 + cnt)
                                '}
                                f+= fstep/t
               f:= fsmax
               debug.Str(String(10,13))

          fminm:= (59*fminm + fsmax)/60     
          debug.Str(String(10, 13, "Fsmax = "))
          debug.Dec(fminm)
          debug.Str(String("  Fmin = "))
          debug.Dec(fsmax)
          debug.Str(String("  Steig = "))
          debug.Dec(steigm)
          if (fmin-fminm)>-2000 and (fmin-fminm)<2000
           repeat (fmin-fminm+2000)*80/3000
            Debug.Str(String("="))
                         
               '   waitcnt(cnt + clkfreq)
        






                  
      "F", "f":              ' Finde Zentralfrequenz
        f-=(11* fstep)        'Zentralfrequenz
        fminm:= f
        fstep*= 3      
        repeat  
          t:= 1
             repeat 7
               t:= t*2
               f-=3*fstep/t
               cmin:= 100000              
                  repeat 6
                                frq.Freq(0, Ausgang, f)   
                                count := phsb
                                waitcnt(clkfreq/300 + cnt)                      
                                count := phsb - count
                                if count<cmin
                                  cmin:=count
                                  fmin:=f
                                {  
                                debug.Str(String(10, 13, "Freq = "))
                                debug.Dec(f)
                                debug.Str(String("  count = "))
                                debug.Dec(count)
                                repeat count*80/500
                                  Debug.Str(String("="))
                                  
                                waitcnt(clkfreq/10 + cnt) }
                                f+= fstep/t
               f:= fmin
               'debug.Str(String(10,13))

          fminm:= (59*fminm + fmin)/60     
          debug.Str(String(10, 13, "Minfm = "))
          debug.Dec(fminm)
          debug.Str(String("  Fmin = "))
          debug.Dec(fmin)
          debug.Str(String("  count = "))
          debug.Dec(cmin)
          if (fmin-fminm)>-1000 and (fmin-fminm)<1000
           repeat (fmin-fminm+1000)*80/1000
            Debug.Str(String("="))
                         
               '   waitcnt(cnt + clkfreq)


                          
      "Q", "q": quit

  Debug.str(String(10, 13, "Bye!"))      